<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * K2 helper class.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Com_K2HelperCom_K2
{
	/**
	 * Template helper
	 *
	 * @var    CsviHelperTemplate
	 * @since  6.0
	 */
	protected $template = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  6.0
	 */
	protected $log = null;

	/**
	 * Fields helper
	 *
	 * @var    CsviHelperFields
	 * @since  6.0
	 */
	protected $fields = null;

	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  6.0
	 */
	protected $db = null;

	/**
	 * Categories list
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $categories = array();

	/**
	 * Category cache
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $categoryCache = array();

	/**
	 * Category options
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $options = array();

	/**
	 * Category separator
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $categorySeparator = null;

	/**
	 * Constructor.
	 *
	 * @param   CsviHelperTemplate  $template  An instance of CsviHelperTemplate.
	 * @param   CsviHelperLog       $log       An instance of CsviHelperLog.
	 * @param   CsviHelperFields    $fields    An instance of CsviHelperFields.
	 * @param   JDatabaseDriver     $db        Database connector.
	 *
	 * @since   4.0
	 */
	public function __construct(CsviHelperTemplate $template, CsviHelperLog $log, CsviHelperFields $fields, JDatabaseDriver $db)
	{
		$this->template = $template;
		$this->log = $log;
		$this->fields = $fields;
		$this->db = $db;
	}

	public function updateAlias($alias)
	{
		$this->fields->set('alias', $alias);
	}

	/**
	 * Get the content id, this is necessary for updating existing content.
	 *
	 * @return  int  The item ID is returned.
	 *
	 * @since   5.3
	 */
	public function getItemId()
	{
		$id = $this->fields->get('id');

		if ($id)
		{
			return $id;
		}
		else
		{
			$alias = $this->fields->get('alias');
			$catid = $this->fields->get('catid', null);

			if (empty($catid))
			{
				$category_path = $this->fields->get('category_path');

				if ($category_path)
				{
					// We have a category path, let's get the ID
					$catid = $this->getCategoryIdByPath($category_path);

					if (empty($catid))
					{
						return false;
					}
				}
				else
				{
					return false;
				}
			}

			if ($alias && $catid)
			{
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('id'))
					->from($this->db->quoteName('#__k2_items'))
					->where($this->db->quoteName('alias') . ' = ' . $this->db->quote($alias))
					->where($this->db->quoteName('catid') . ' = ' . (int) $catid);
				$this->db->setQuery($query);
				$this->log->add('COM_CSVI_FIND_ITEM_ID');

				return $this->db->loadResult();
			}
			else
			{
				return false;
			}
		}
	}

	/**
	 * Get the category ID based on it's path.
	 *
	 * @param   string  $item_id  The path of the category
	 *
	 * @return  int  The ID of the category.
	 *
	 * @since   3.0
	 */
	public function getCategoryId($item_id)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('catid'))
			->from($this->db->quoteName('#__k2_items'))
			->where($this->db->quoteName('id') . ' = ' . (int) $item_id);
		$this->db->setQuery($query, 0, 1);

		return $this->db->loadResult();
	}

	/**
	 * Get the category tree.
	 *
	 * @param   string  $language  The language code for the category names.
	 *
	 * @return  array  Category tree parts.
	 *
	 * @since   4.0
	 */
	public function getCategoryTree($language)
	{
		// 1. Get all categories
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName('c.parent', 'parent_id') . ',' .
				$this->db->quoteName('c.id') . ',' .
				$this->db->quoteName('c.name', 'catname')
			)
			->from($this->db->quoteName('#__k2_categories', 'c'))
			->leftJoin(
				$this->db->quoteName('#__k2_categories', 'x')
				. ' ON ' . $this->db->quoteName('x.parent') . ' = ' . $this->db->quoteName('c.id')
			)
			->where($this->db->quoteName('c.language') . ' = ' . $this->db->quote($language));
		$this->db->setQuery($query);
		$rawcats = $this->db->loadObjectList();

		if (!empty($rawcats))
		{
			// 2. Group categories based on their parent_id
			$categories = array();

			foreach ($rawcats as $rawcat)
			{
				$categories[$rawcat->parent_id][$rawcat->id]['pid'] = $rawcat->parent_id;
				$categories[$rawcat->parent_id][$rawcat->id]['cid'] = $rawcat->id;
				$categories[$rawcat->parent_id][$rawcat->id]['catname'] = $rawcat->catname;
			}
		}

		$this->options = array();

		if (isset($categories))
		{
			if (count($categories) > 0)
			{
				// Take the toplevels first
				foreach ($categories[0] as $category)
				{
					$this->options[] = JHtml::_('select.option', $category['cid'], $category['catname']);

					// Write the subcategories
					$this->buildCategory($categories, $category['cid']);
				}
			}
		}

		return $this->options;
	}

	/**
	 * Create a category path based on ID.
	 *
	 * @param   array  $catids  list of IDs to generate category path for.
	 *
	 * @return  string  The category paths.
	 *
	 * @since   4.0
	 */
	public function createCategoryPathById($catids)
	{
		if (!is_array($catids))
		{
			$catids = (array) $catids;
		}

		$paths = $this->constructCategoryPath($catids);

		if (is_array($paths))
		{
			return implode('|', $paths);
		}
		else
		{
			return '';
		}
	}

	/**
	 * Get the ID belonging to the category path.
	 *
	 * @param   string  $category_path  The path to get the ID for.
	 * @param   string  $language       The language of the path.
	 *
	 * @return  int  The ID of the category.
	 *
	 * @since   3.0
	 */
	public function getCategoryIdByPath($category_path, $language='*')
	{
		// Check for any missing categories, otherwise create them
		$category = $this->processCategory($category_path, $language);

		return end($category);
	}

	/**
	 * Find the ID of an extra field.
	 *
	 * @return  int  The ID of the extra field.
	 *
	 * @since   6.0
	 */
	public function getExtrafieldId()
	{
		$id = $this->fields->get('id');

		if ($id)
		{
			return $id;
		}
		else
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('id'))
				->from($this->db->quoteName('#__k2_extra_fields'))
				->where($this->db->quoteName('name') . ' = ' . $this->db->quote($this->fields->get('name')));
			$this->db->setQuery($query);
			$this->log->add('COM_CSVI_FIND_EXTRAFIELDS_ID');

			return $this->db->loadResult();
		}
	}

	/**
	 * Create the subcategory layout.
	 *
	 * @param   array   $cattree    An array of categories to get the tree for.
	 * @param   string  $catfilter  A filter for the categories.
	 * @param   int     $loop       The position we are at.
	 *
	 * @return  void.
	 *
	 * @since   3.0
	 */
	private function buildCategory($cattree, $catfilter, $loop=1)
	{
		if (isset($cattree[$catfilter]))
		{
			foreach ($cattree[$catfilter] as $subcatid => $category)
			{
				$this->options[] = JHtml::_('select.option', $category['cid'], str_repeat('>', $loop) . ' ' . $category['catname']);
				$this->buildCategory($cattree, $subcatid, $loop + 1);
			}
		}
	}

	/**
	 * Construct the category path.
	 *
	 * @param   array  $catids  The IDs to build a category for.
	 *
	 * @return  array  List of category paths.
	 *
	 * @since   4.0
	 */
	private function constructCategoryPath($catids)
	{
		$catpaths = array();

		if (is_array($catids))
		{
			// Load the category separator
			if (is_null($this->categorySeparator))
			{
				$this->categorySeparator = $this->template->get('category_separator', '/');
			}

			// Get the paths
			foreach ($catids as $category_id)
			{
				// Create the path
				$paths = array();

				while ($category_id > 0)
				{
					$query = $this->db->getQuery(true)
						->select($this->db->quoteName('c.parent') . ',' . $this->db->quoteName('c.name'))
						->from($this->db->quoteName('#__k2_categories', 'c'))
						->leftJoin(
							$this->db->quoteName('#__k2_categories', 'x')
							. ' ON ' . $this->db->quoteName('x.parent') . ' = ' . $this->db->quoteName('c.id')
						)
						->where($this->db->quoteName('c.id') . ' = ' . (int) $category_id)
						->where($this->db->quoteName('c.language') . ' = ' . $this->db->quote($this->template->get('language', '*')));
					$this->db->setQuery($query);
					$path = $this->db->loadObject();
					$this->log->add('Get cat ID ' . $category_id);

					if (is_object($path))
					{
						$paths[] = $path->name;
						$category_id = $path->parent;
					}
					else
					{
						$this->log->add(JText::sprintf('COM_CSVI_CANNOT_GET_CATEGORY_PATH', $category_id, $this->template->get('language', '*')));
						$this->log->addStats('incorrect', JText::sprintf('COM_CSVI_CANNOT_GET_CATEGORY_PATH', $category_id, $this->template->get('language', '*')));

						return '';
					}
				}

				// Create the path
				$paths = array_reverse($paths);
				$catpaths[] = implode($this->categorySeparator, $paths);
			}
		}

		return $catpaths;
	}

	/**
	 * Creates categories.
	 *
	 * @param   array   $category_path  Contains the category/categories for an item.
	 * @param   string  $language       The language the category path is in.
	 *
	 * @return  array  List of category IDs.
	 *
	 * @since   6.0
	 */
	private function processCategory($category_path, $language='*')
	{
		// Load the category separator
		if (is_null($this->categorySeparator))
		{
			$this->categorySeparator = $this->template->get('category_separator', '/');
		}

		// Load the categories table
		require_once JPATH_PLUGINS . '/csviaddon/k2/com_k2/table/category.php';
		$options = array(
			'template' => $this->template,
			'log' => $this->log
		);
		$this->categories = new K2TableCategory('', '', $this->db, $options);

		// Load the transliterate helper
		$translit = new CsviHelperTranslit($this->template);

		$this->log->add('Checking category path: ' . $category_path, false);

		// Explode slash delimited category tree into array
		$category_list = explode($this->categorySeparator, $category_path);
		$category_count = count($category_list);

		$category = array();
		$category_parent_id = '0';

		// For each category in array
		for ($i = 0; $i < $category_count; $i++)
		{
			// Check the cache first
			if (array_key_exists($category_parent_id . '.' . $category_list[$i], $this->categoryCache))
			{
				$category_id = $this->categoryCache[$category_parent_id . '.' . $category_list[$i]];
			}
			else
			{
				// See if this category exists with it's parent in xref
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('c.id'))
					->from('#__k2_categories c')
					->leftJoin(
						$this->db->quoteName('#__k2_categories', 'x')
						. ' ON ' . $this->db->quoteName('x.parent') . ' = ' . $this->db->quoteName('c.id')
					)
					->where($this->db->quoteName('c.name') . ' = ' . $this->db->quote($category_list[$i]))
					->where($this->db->quoteName('c.parent') . ' = ' . (int) $category_parent_id);
				$this->db->setQuery($query);
				$category_id = $this->db->loadResult();
				$this->log->add('Check if the category exists');

				// Add result to cache
				$this->categoryCache[$category_parent_id . '.' . $category_list[$i]] = $category_id;
			}

			// Category does not exist - create it
			if (is_null($category_id) && !$this->template->get('ignore_non_exist', false))
			{
				// Let's find out the last category in the level of the new category
				$query = $this->db->getQuery(true)
					->select('MAX(' . $this->db->quoteName('c.ordering') . ') + 1 AS ' . $this->db->quoteName('ordering'))
					->from($this->db->quoteName('#__k2_categories', 'c'))
					->leftJoin(
						$this->db->quoteName('#__k2_categories', 'x')
						. ' ON ' . $this->db->quoteName('x.parent') . ' = ' . $this->db->quoteName('c.id')
					)
					->where($this->db->quoteName('x.parent') . ' = ' . (int) $category_parent_id);
				$this->db->setQuery($query);
				$list_order = $this->db->loadResult();

				if (is_null($list_order))
				{
					$list_order = 1;
				}

				// Add category
				$this->categories->set('name', $category_list[$i]);

				// Create the alias
				$alias = $translit->stringURLSafe($category_list[$i]);

				$this->categories->set('alias', $alias);
				$this->categories->set('access', 1);
				$this->categories->set('params', '{"inheritFrom":"0","theme":"","num_leading_items":"2","num_leading_columns":"1","leadingImgSize":"Large","num_primary_items":"4","num_primary_columns":"2","primaryImgSize":"Medium","num_secondary_items":"4","num_secondary_columns":"1","secondaryImgSize":"Small","num_links":"4","num_links_columns":"1","linksImgSize":"XSmall","catCatalogMode":"0","catFeaturedItems":"1","catOrdering":"","catPagination":"2","catPaginationResults":"1","catTitle":"1","catTitleItemCounter":"1","catDescription":"1","catImage":"1","catFeedLink":"1","catFeedIcon":"1","subCategories":"1","subCatColumns":"2","subCatOrdering":"","subCatTitle":"1","subCatTitleItemCounter":"1","subCatDescription":"1","subCatImage":"1","itemImageXS":"","itemImageS":"","itemImageM":"","itemImageL":"","itemImageXL":"","catItemTitle":"1","catItemTitleLinked":"1","catItemFeaturedNotice":"0","catItemAuthor":"1","catItemDateCreated":"1","catItemRating":"0","catItemImage":"1","catItemIntroText":"1","catItemIntroTextWordLimit":"","catItemExtraFields":"0","catItemHits":"0","catItemCategory":"1","catItemTags":"1","catItemAttachments":"0","catItemAttachmentsCounter":"0","catItemVideo":"0","catItemVideoWidth":"","catItemVideoHeight":"","catItemAudioWidth":"","catItemAudioHeight":"","catItemVideoAutoPlay":"0","catItemImageGallery":"0","catItemDateModified":"0","catItemReadMore":"1","catItemCommentsAnchor":"1","catItemK2Plugins":"1","itemDateCreated":"1","itemTitle":"1","itemFeaturedNotice":"1","itemAuthor":"1","itemFontResizer":"1","itemPrintButton":"1","itemEmailButton":"1","itemSocialButton":"1","itemVideoAnchor":"1","itemImageGalleryAnchor":"1","itemCommentsAnchor":"1","itemRating":"1","itemImage":"1","itemImgSize":"Large","itemImageMainCaption":"1","itemImageMainCredits":"1","itemIntroText":"1","itemFullText":"1","itemExtraFields":"1","itemDateModified":"1","itemHits":"1","itemCategory":"1","itemTags":"1","itemAttachments":"1","itemAttachmentsCounter":"1","itemVideo":"1","itemVideoWidth":"","itemVideoHeight":"","itemAudioWidth":"","itemAudioHeight":"","itemVideoAutoPlay":"0","itemVideoCaption":"1","itemVideoCredits":"1","itemImageGallery":"1","itemNavigation":"1","itemComments":"1","itemTwitterButton":"1","itemFacebookButton":"1","itemGooglePlusOneButton":"1","itemAuthorBlock":"1","itemAuthorImage":"1","itemAuthorDescription":"1","itemAuthorURL":"1","itemAuthorEmail":"0","itemAuthorLatest":"1","itemAuthorLatestLimit":"5","itemRelated":"1","itemRelatedLimit":"5","itemRelatedTitle":"1","itemRelatedCategory":"0","itemRelatedImageSize":"0","itemRelatedIntrotext":"0","itemRelatedFulltext":"0","itemRelatedAuthor":"0","itemRelatedMedia":"0","itemRelatedImageGallery":"0","itemK2Plugins":"1","catMetaDesc":"","catMetaKey":"","catMetaRobots":"","catMetaAuthor":""}');
				$this->categories->set('parent', $category_parent_id);
				$this->categories->set('ordering', $list_order);
				$this->categories->set('published', 1);
				$this->categories->set('language', $language);
				$this->categories->store();
				$this->log->add('Add new category', false);
				$category_id = $this->categories->get('id');

				// Add result to cache
				$this->categoryCache[$category_parent_id . '.' . $category_list[$i]] = $category_id;

				// Clean for the next row
				$this->categories->reset();
			}
			elseif ($this->template->get('ignore_non_exist', false))
			{
				return array(0);
			}

			// Set this category as parent of next in line
			$category_parent_id = $category_id;
			$category[] = $category_id;
		}

		// Return an array with the last category_ids which is where the product goes
		return $category;
	}

	/**
	 * Get the k2 item tags.
	 *
	 * @return  array  K2 item tags.
	 *
	 * @since   6.4.0
	 */
	public function getItemTags()
	{
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName('name') . ',' .
				$this->db->quoteName('id')
			)
			->from($this->db->quoteName('#__k2_tags'))
			->where($this->db->quoteName('published') . ' = 1');
		$this->db->setQuery($query);
		$itemTags = $this->db->loadObjectList();

		$this->options = array();

		if (!empty($itemTags))
		{
			foreach ($itemTags as $tag)
			{
				$this->options[] = JHtml::_('select.option', $tag->id, $tag->name);
			}
		}

		return $this->options;
	}

	/**
	 * Get the list for author filter.
	 *
	 * @return  array  users list.
	 *
	 * @since   6.4.0
	 */
	public function getAuthors()
	{
		$query = $this->db->getQuery(true)
		->select(
				$this->db->quoteName('u.name') . ',' .
				$this->db->quoteName('u.id')
			)
			->from($this->db->quoteName('#__users', 'u'))
			->innerJoin(
				$this->db->quoteName('#__k2_items', 'i')
				. ' ON ' . $this->db->quoteName('i.created_by') . ' = ' . $this->db->quoteName('u.id')
			)
			->where($this->db->quoteName('u.block') . ' = 0')
			->group($this->db->quoteName('i.created_by'));
		$this->db->setQuery($query);
		$users = $this->db->loadObjectList();

		$this->options = array();

		if (!empty($users))
		{
			foreach ($users as $user)
			{
				$this->options[] = JHtml::_('select.option', $user->id, $user->name);
			}
		}

		return $this->options;
	}

	/**
	 * Check if a content plugin exists.
	 *
	 * @param   string  $plugin  The name of the plugin to check.
	 *
	 * @return  bool  True if exists | False on failure.
	 *
	 * @since   6.0
	 */
	public function pluginExists($plugin)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('extension_id'))
			->from($this->db->quoteName('#__extensions'))
			->where($this->db->quoteName('name') . ' = ' . $this->db->quote($plugin))
			->where($this->db->quoteName('type') . ' = ' . $this->db->quote('plugin'))
			->where($this->db->quoteName('folder') . ' = ' . $this->db->quote('content'));
		$this->db->setQuery($query);

		return $this->db->loadResult();
	}

	/**
	 * Get the extra field group ID based on catid.
	 *
	 * @param   string  $catId  The ID of the category
	 *
	 * @return  int  The ID of the extra fields group.
	 *
	 * @since   8.4.0
	 */
	public function getExtraFieldGroup(int $catId): int
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('extraFieldsGroup'))
			->from($this->db->quoteName('#__k2_categories'))
			->where($this->db->quoteName('id') . ' = ' . $catId);
		$this->db->setQuery($query, 0, 1);

		return $this->db->loadResult();
	}

	/**
	 * Get the extra field group name based on id.
	 *
	 * @param   string  $groupId  The ID of the group
	 *
	 * @return  string  The name of the extra fields group.
	 *
	 * @since   8.4.0
	 */
	public function getExtraFieldGroupName(int $groupId): string
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('name'))
			->from($this->db->quoteName('#__k2_extra_fields_groups'))
			->where($this->db->quoteName('id') . ' = ' . $groupId);
		$this->db->setQuery($query, 0, 1);

		return $this->db->loadResult();
	}

	/**
	 * Get the category alias based on id.
	 *
	 * @param   string  $catid  The ID of the category
	 *
	 * @return  string  The alias of the category.
	 *
	 * @since   8.6.0
	 */
	public function getCategoryAlias(int $catid): string
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('alias'))
			->from($this->db->quoteName('#__k2_categories'))
			->where($this->db->quoteName('id') . ' = ' . $catid);
		$this->db->setQuery($query);

		return $this->db->loadResult();
	}
}
